@@ -28,6 +28,7 @@
 #include "legacy.h"
 #include "GenericBdsLib.h"
 #include "../refind/global.h"
+#include "../refind/lib.h"
 #include "../include/refit_call_wrapper.h"
 
 BOOT_OPTION_BBS_MAPPING  *mBootOptionBbsMapping     = NULL;
@@ -181,9 +182,9 @@ BdsBuildLegacyDevNameString (
   //
   if (Index >= 5 && Index <= 16 && (CurBBSEntry->DeviceType == BBS_HARDDISK || CurBBSEntry->DeviceType == BBS_CDROM)) {
     Fmt = L"%s %d";
-    UnicodeSPrint (BootString, BufSize, Fmt, Type, Index - 5);
+    SPrint(BootString, BufSize, Fmt, Type, Index - 5);
   } else {
-    UnicodeSPrint (BootString, BufSize, Fmt, Type);
+    SPrint(BootString, BufSize, Fmt, Type);
   }
 }
 
@@ -274,7 +275,7 @@ BdsFindLegacyBootOptionByDevTypeAndName (
   // Loop all boot option from variable
   //
   for (Index = 0; Index < BootOptionNum; Index++) {
-    UnicodeSPrint (BootOption, sizeof (BootOption), L"Boot%04x", (UINTN) BootOrder[Index]);
+    SPrint(BootOption, sizeof (BootOption), L"Boot%04x", (UINTN) BootOrder[Index]);
     BootOptionVar = BdsLibGetVariableAndSize (
                       BootOption,
                       &EfiGlobalVariableGuid,
@@ -379,7 +380,7 @@ BdsCreateLegacyBootOption (
     CurrentBootOptionNo = (UINT16) ArrayIndex;
   }
 
-  UnicodeSPrint (
+  SPrint (
     BootString,
     sizeof (BootString),
     L"Boot%04x",
@@ -460,15 +461,9 @@ BdsCreateLegacyBootOption (
 
   Ptr += sizeof (BBS_TABLE);
   *((UINT16 *) Ptr) = (UINT16) Index;
-
-  Status = refit_call5_wrapper(gRT->SetVariable,
-                  BootString,
-                  &EfiGlobalVariableGuid,
-                  VAR_FLAG,
-                  BufferSize,
-                  Buffer
-                  );
-
+  Status = EfivarSetRaw(&EfiGlobalVariableGuid, BootString,
+                        (CHAR8*) Buffer,
+                        BufferSize, TRUE);
   FreePool (Buffer);
   
   Buffer = NULL;
@@ -816,13 +811,9 @@ BdsAddNonExistingLegacyBootOptions (
     );
 
   if (BootOrderSize > 0) {
-    Status = refit_call5_wrapper(gRT->SetVariable,
-                    L"BootOrder",
-                    &EfiGlobalVariableGuid,
-                    VAR_FLAG,
-                    BootOrderSize,
-                    BootOrder
-                    );
+    Status = EfivarSetRaw(&EfiGlobalVariableGuid, L"BootOrder",
+                          (CHAR8*) BootOrder,
+                          BootOrderSize, TRUE);
   } else {
     EfiLibDeleteVariable (L"BootOrder", &EfiGlobalVariableGuid);
   }
@@ -861,7 +852,7 @@ BdsDeleteBootOption (
   Status    = EFI_SUCCESS;
   Index2Del = 0;
 
-  UnicodeSPrint (BootOption, sizeof (BootOption), L"Boot%04x", OptionNumber);
+  SPrint (BootOption, sizeof (BootOption), L"Boot%04x", OptionNumber);
   Status = EfiLibDeleteVariable (BootOption, &EfiGlobalVariableGuid);
 
   //
@@ -950,7 +941,7 @@ BdsDeleteAllInvalidLegacyBootOptions (
 
   Index = 0;
   while (Index < BootOrderSize / sizeof (UINT16)) {
-    UnicodeSPrint (BootOption, sizeof (BootOption), L"Boot%04x", BootOrder[Index]);
+    SPrint (BootOption, sizeof (BootOption), L"Boot%04x", BootOrder[Index]);
     BootOptionVar = BdsLibGetVariableAndSize (
                       BootOption,
                       &EfiGlobalVariableGuid,
@@ -1034,13 +1025,9 @@ BdsDeleteAllInvalidLegacyBootOptions (
   // Adjust the number of boot options.
   //
   if (BootOrderSize != 0) {
-    Status = refit_call5_wrapper(gRT->SetVariable,
-                    L"BootOrder",
-                    &EfiGlobalVariableGuid,
-                    VAR_FLAG,
-                    BootOrderSize,
-                    BootOrder
-                    );
+    Status = EfivarSetRaw(&EfiGlobalVariableGuid, L"BootOrder",
+                          (CHAR8*) BootOrder,
+                          BootOrderSize, TRUE);
   } else {
     EfiLibDeleteVariable (L"BootOrder", &EfiGlobalVariableGuid);
   }
