@@ -24,15 +24,16 @@
 #include "lib.h"
 #include "screen.h"
 #include "apple.h"
+#include "log.h"
 #include "mystrings.h"
 #include "../include/refit_call_wrapper.h"
 
-CHAR16 gCsrStatus[256];
+CHAR16 gCsrStatus[512];
 
-// Get CSR (Apple's System Integrity Protection [SIP], or "rootless") status
-// information. If the variable is not present and the firmware is Apple, fake
-// it and claim it's enabled, since that's how OS X 10.11 treats a system with
-// the variable absent.
+// Get CSR (Apple's Configurable Security Restrictions; aka System Integrity
+// Protection [SIP], or "rootless") status information. If the variable is not
+// present and the firmware is Apple, fake it and claim it's enabled, since
+// that's how OS X 10.11 treats a system with the variable absent.
 EFI_STATUS GetCsrStatus(UINT32 *CsrStatus) {
     UINT32     *ReturnValue = NULL;
     UINTN      CsrLength;
@@ -67,7 +68,11 @@ VOID RecordgCsrStatus(UINT32 CsrStatus, BOOLEAN DisplayMessage) {
         case SIP_ENABLED:
             SPrint(gCsrStatus, 255, L" System Integrity Protection is enabled (0x%02x)", CsrStatus);
             break;
-        case SIP_DISABLED:
+        case SIP_DISABLED_10:
+            SPrint(gCsrStatus, 255, L" System Integrity Protection is disabled for OS X 10.x (0x%02x)",
+                   CsrStatus);
+            break;
+        case SIP_DISABLED_11:
             SPrint(gCsrStatus, 255, L" System Integrity Protection is disabled (0x%02x)", CsrStatus);
             break;
         default:
@@ -88,6 +93,7 @@ VOID RotateCsrValue(VOID) {
     EFI_GUID     CsrGuid = CSR_GUID;
     EFI_STATUS   Status;
 
+    LOG(1, LOG_LINE_SEPARATOR, L"Rotating CSR value");
     Status = GetCsrStatus(&CurrentValue);
     if ((Status == EFI_SUCCESS) && GlobalConfig.CsrValues) {
         ListItem = GlobalConfig.CsrValues;
@@ -98,12 +104,18 @@ VOID RotateCsrValue(VOID) {
         } else {
             TargetCsr = ListItem->Next->Value;
         }
+        LOG(1, LOG_LINE_NORMAL, L"CSR value was 0x%04x; setting to 0x%04x", CurrentValue, TargetCsr);
         Status = EfivarSetRaw(&CsrGuid, L"csr-active-config", (CHAR8 *) &TargetCsr, 4, TRUE);
-        if (Status == EFI_SUCCESS)
+        if (Status == EFI_SUCCESS) {
             RecordgCsrStatus(TargetCsr, TRUE);
-        else
+            LOG(2, LOG_LINE_NORMAL, L"Successful setting of CSR value of 0x%04x", TargetCsr);
+        } else {
             SPrint(gCsrStatus, 255, L" Error setting System Integrity Protection code.");
-    } // if
+            LOG(1, LOG_LINE_NORMAL, gCsrStatus);
+        }
+    } else {
+        LOG(1, LOG_LINE_NORMAL, L"Could not retrieve CSR value or csr_values not set");
+    } // if/else
 } // VOID RotateCsrValue()
 
 
@@ -135,16 +147,20 @@ EFI_STATUS SetAppleOSInfo() {
     EFI_GUID apple_set_os_guid = EFI_APPLE_SET_OS_PROTOCOL_GUID;
     EfiAppleSetOsInterface *SetOs = NULL;
 
+    LOG(1, LOG_LINE_NORMAL, L"Setting Apple OS information, if applicable");
     Status = refit_call3_wrapper(BS->LocateProtocol, &apple_set_os_guid, NULL, (VOID**) &SetOs);
 
     // If not a Mac, ignore the call....
-    if ((Status != EFI_SUCCESS) || (!SetOs))
+    if ((Status != EFI_SUCCESS) || (!SetOs)) {
+        LOG(2, LOG_LINE_NORMAL, L"Not a Mac; not setting Apple OS information");
         return EFI_SUCCESS;
+    }
 
     if ((SetOs->Version != 0) && GlobalConfig.SpoofOSXVersion) {
         AppleOSVersion = StrDuplicate(L"Mac OS X");
         MergeStrings(&AppleOSVersion, GlobalConfig.SpoofOSXVersion, ' ');
         if (AppleOSVersion) {
+            LOG(2, LOG_LINE_NORMAL, L"Setting Apple OS information to '%s'", AppleOSVersion);
             AppleOSVersion8 = AllocateZeroPool((StrLen(AppleOSVersion) + 1) * sizeof(CHAR8));
             UnicodeStrToAsciiStr(AppleOSVersion, AppleOSVersion8);
             if (AppleOSVersion8) {
@@ -165,4 +181,4 @@ EFI_STATUS SetAppleOSInfo() {
         Print(L"Unable to set firmware boot type!\n");
 
     return (Status);
-} // EFI_STATUS SetAppleOSInfo()
\ No newline at end of file
+} // EFI_STATUS SetAppleOSInfo()
