@@ -34,7 +34,7 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 /*
- * Modifications copyright (c) 2012-2020 Roderick W. Smith
+ * Modifications copyright (c) 2012-2021 Roderick W. Smith
  *
  * Modifications distributed under the terms of the GNU General Public
  * License (GPL) version 3 (GPLv3), a copy of which must be distributed
@@ -64,25 +64,26 @@
 #define TAG_TOOL             (4)
 #define TAG_LOADER           (5)
 #define TAG_LEGACY           (6)
-#define TAG_EXIT             (7)
-#define TAG_SHELL            (8)
-#define TAG_GPTSYNC          (9)
-#define TAG_LEGACY_UEFI      (10)
-#define TAG_APPLE_RECOVERY   (11)
-#define TAG_WINDOWS_RECOVERY (12)
-#define TAG_MOK_TOOL         (13)
-#define TAG_FIRMWARE         (14)
-#define TAG_MEMTEST          (15)
-#define TAG_GDISK            (16)
-#define TAG_NETBOOT          (17)
-#define TAG_CSR_ROTATE       (18)
-#define TAG_FWUPDATE_TOOL    (19)
-#define TAG_HIDDEN           (20)
-#define TAG_INSTALL          (21)
-#define TAG_BOOTORDER        (22)
-#define NUM_TOOLS            (23)
+#define TAG_FIRMWARE_LOADER  (7)
+#define TAG_EXIT             (8)
+#define TAG_SHELL            (9)
+#define TAG_GPTSYNC          (10)
+#define TAG_LEGACY_UEFI      (11)
+#define TAG_APPLE_RECOVERY   (12)
+#define TAG_WINDOWS_RECOVERY (13)
+#define TAG_MOK_TOOL         (14)
+#define TAG_FIRMWARE         (15)
+#define TAG_MEMTEST          (16)
+#define TAG_GDISK            (17)
+#define TAG_NETBOOT          (18)
+#define TAG_CSR_ROTATE       (19)
+#define TAG_FWUPDATE_TOOL    (20)
+#define TAG_HIDDEN           (21)
+#define TAG_INSTALL          (22)
+#define TAG_BOOTORDER        (23)
+#define NUM_TOOLS            (24)
 
-#define NUM_SCAN_OPTIONS 10
+#define NUM_SCAN_OPTIONS 11
 
 #define DEFAULT_ICONS_DIR L"icons"
 
@@ -138,8 +139,9 @@
 #define FS_TYPE_REISERFS       9
 #define FS_TYPE_BTRFS          10
 #define FS_TYPE_XFS            11
-#define FS_TYPE_ISO9660        12
-#define NUM_FS_TYPES           13
+#define FS_TYPE_JFS            12
+#define FS_TYPE_ISO9660        13
+#define NUM_FS_TYPES           14
 
 // How to scale banner images
 #define BANNER_NOSCALE         0
@@ -221,9 +223,12 @@
 #define NULL_GUID_VALUE { 0x00000000, 0x0000, 0x0000, {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00} };
 #define REFIND_GUID_VALUE { 0x36D08FA7, 0xCF0B, 0x42F5, {0x8F, 0x14, 0x68, 0xDF, 0x73, 0xED, 0x37, 0x40} };
 #define ESP_GUID_VALUE { 0xc12a7328, 0xf81f, 0x11d2, { 0xba, 0x4b, 0x00, 0xa0, 0xc9, 0x3e, 0xc9, 0x3b } };
+#define SYSTEMD_GUID_VALUE { 0x4a67b082, 0x0a4c, 0x41cf, { 0xb6, 0xc7, 0x44, 0x0b, 0x29, 0xbb, 0x8c, 0x4f }};
+
 
 // Configuration file variables
 #define KERNEL_VERSION L"%v"
+#define MAX_RES_CODE 2147483647 /* 2^31 - 1 */
 
 //
 // global definitions
@@ -253,8 +258,9 @@ typedef struct {
    EFI_DEVICE_PATH     *DevicePath;
    EFI_HANDLE          DeviceHandle;
    EFI_FILE            *RootDir;
-   CHAR16              *VolName;
-   CHAR16              *PartName;
+   CHAR16              *PartName; // GPT partition name
+   CHAR16              *FsName;   // Filesystem name
+   CHAR16              *VolName;  // One of the two above OR fs description (e.g., "2 GiB FAT volume")
    EFI_GUID            VolUuid;
    EFI_GUID            PartGuid;
    EFI_GUID            PartTypeGuid;
@@ -288,7 +294,7 @@ typedef struct _refit_menu_entry {
 } REFIT_MENU_ENTRY;
 
 typedef struct _refit_menu_screen {
-   CHAR16      *Title;
+   CHAR16      *Title; // For EFI firmware entry, this includes "Reboot to" prefix
    EG_IMAGE    *TitleImage;
    UINTN       InfoLineCount;
    CHAR16      **InfoLines;
@@ -302,7 +308,7 @@ typedef struct _refit_menu_screen {
 
 typedef struct {
    REFIT_MENU_ENTRY me;
-   CHAR16           *Title;
+   CHAR16           *Title; // For EFI firmware entry, this is "raw" title
    CHAR16           *LoaderPath;
    REFIT_VOLUME     *Volume;
    BOOLEAN          UseGraphicsMode;
@@ -311,6 +317,8 @@ typedef struct {
    CHAR16           *InitrdPath; // Linux stub loader only
    CHAR8            OSType;
    UINTN            DiscoveryType;
+   EFI_DEVICE_PATH  *EfiLoaderPath; // path to NVRAM-defined loader
+   UINT16           EfiBootNum; // Boot#### number for NVRAM-defined loader
 } LOADER_ENTRY;
 
 typedef struct {
@@ -333,6 +341,7 @@ typedef struct {
    BOOLEAN          UseNvram;
    BOOLEAN          ShutdownAfterTimeout;
    BOOLEAN          Install;
+   BOOLEAN          WriteSystemdVars;
    UINTN            RequestedScreenWidth;
    UINTN            RequestedScreenHeight;
    UINTN            BannerBottomEdge;
@@ -347,6 +356,7 @@ typedef struct {
    UINTN            MouseSpeed;
    UINTN            IconSizes[4];
    UINTN            BannerScale;
+   UINTN            LogLevel;
    REFIT_VOLUME     *DiscoveredRoot;
    EFI_DEVICE_PATH  *SelfDevicePath;
    CHAR16           *BannerFileName;
@@ -360,6 +370,7 @@ typedef struct {
    CHAR16           *DontScanDirs;
    CHAR16           *DontScanFiles;
    CHAR16           *DontScanTools;
+   CHAR16           *DontScanFirmware;
    CHAR16           *WindowsRecoveryFiles;
    CHAR16           *MacOSRecoveryFiles;
    CHAR16           *DriverDirs;
@@ -396,6 +407,8 @@ extern EFI_GUID RefindGuid;
 extern REFIT_MENU_SCREEN MainMenu;
 extern REFIT_MENU_ENTRY MenuEntryReturn;
 
+// Global function definitions....
+
 VOID AboutrEFInd(VOID);
 EG_IMAGE * GetDiskBadge(IN UINTN DiskType);
 LOADER_ENTRY * MakeGenericLoaderEntry(VOID);
